@@ -101,7 +101,7 @@ const paths = {
 
 
 const tsBaseArgs = {
-  target: "es5",
+  target: "es6",
   jsx: "react",
   reactNamespace: "React",
   experimentalDecorators: true,
@@ -166,9 +166,14 @@ gulp.task("dist-prod", ["compile-prod"], function () {
 gulp.task("compile-prod", function (callback) {
   let config = require("./webpack.config.js")({prod: true});
   webpack(config, function(err, stats) {
-    if(err) {
+    if (err) {
       throw new gutil.PluginError("webpack", err);
     }
+    if (stats.hasErrors() || stats.hasWarnins) {
+      gutil.log("[webpack]", stats.toString({
+        colors: true,
+      }));
+    }
     callback();
   });
 });
