@@ -106,7 +106,7 @@ export interface QueryStream<T> {
   /**
    * Reduce the stream, resulting in a single value.
    */
-  reduce<S>(f: (v: T, acc: S) => S, start?: S): Promise<S>;
+  reduce<S>(f: (v: T, acc?: S) => S, start?: S): Promise<S>;
 
   /**
    * Map each element of the stream using a function, resulting in another
@@ -130,7 +130,11 @@ export interface QueryStream<T> {
    */
   first(): QueryValue<T>;
 
-  then(onfulfill: any, onreject: any): any;
+  /**
+   * Run the query without returning a result.
+   * Useful for queries with side effects.
+   */
+  run(): Promise<void>;
 }
 
 
@@ -225,7 +229,7 @@ export const AbortTransaction = Symbol("abort_transaction");
  * function.
  */
 export function openPromise<T>(): any {
-  let resolve: ((value?: T | PromiseLike<T>) => void) | null = null;
+  let resolve: ((x?: any) => void) | null = null;
   let reject: ((reason?: any) => void) | null = null;
   const promise = new Promise<T>((res, rej) => {
     resolve = res;
@@ -239,7 +243,7 @@ export function openPromise<T>(): any {
 }
 
 
-abstract class QueryStreamBase<T> implements QueryStream<T>, PromiseLike<void> {
+abstract class QueryStreamBase<T> implements QueryStream<T> {
   abstract subscribe(f: (isDone: boolean,
                          value: any,
                          tx: IDBTransaction) => void): void;
@@ -250,11 +254,6 @@ abstract class QueryStreamBase<T> implements QueryStream<T>, PromiseLike<void> {
     return new FirstQueryValue(this);
   }
 
-  then<R>(onfulfilled: (value: void) => R | PromiseLike<R>,
-          onrejected: (reason: any) => R | PromiseLike<R>): PromiseLike<R>  {
-    return this.root.then(onfulfilled, onrejected);
-  }
-
   flatMap<S>(f: (x: T) => S[]): QueryStream<S> {
     return new QueryStreamFlatMap<T, S>(this, f);
   }
@@ -318,6 +317,21 @@ abstract class QueryStreamBase<T> implements QueryStream<T>, PromiseLike<void> {
                   .then(() => this.root.finish())
                   .then(() => promise);
   }
+
+  run(): Promise<void> {
+    const {resolve, promise} = openPromise();
+
+    this.subscribe((isDone, value) => {
+      if (isDone) {
+        resolve();
+        return;
+      }
+    });
+
+    return Promise.resolve()
+                  .then(() => this.root.finish())
+                  .then(() => promise);
+  }
 }
 
 type FilterFn = (e: any) => boolean;
@@ -519,7 +533,7 @@ class IterQueryStream<T> extends QueryStreamBase<T> {
 /**
  * Root wrapper around an IndexedDB for queries with a fluent interface.
  */
-export class QueryRoot implements PromiseLike<void> {
+export class QueryRoot {
   private work: Array<((t: IDBTransaction) => void)> = [];
   private stores = new Set();
   private kickoffPromise: Promise<void>;
@@ -534,12 +548,16 @@ export class QueryRoot implements PromiseLike<void> {
 
   private finished: boolean = false;
 
+  private keys: { [keyName: string]: IDBValidKey } = {};
+
   constructor(public db: IDBDatabase) {
   }
 
-  then<R>(onfulfilled: (value: void) => R | PromiseLike<R>,
-          onrejected: (reason: any) => R | PromiseLike<R>): PromiseLike<R> {
-    return this.finish().then(onfulfilled, onrejected);
+  /**
+   * Get a named key that was created during the query.
+   */
+  key(keyName: string): IDBValidKey|undefined {
+    return this.keys[keyName];
   }
 
   private checkFinished() {
@@ -619,10 +637,15 @@ export class QueryRoot implements PromiseLike<void> {
    * Overrides if an existing object with the same key exists
    * in the store.
    */
-  put<T>(store: Store<T>, val: T): QueryRoot {
+  put<T>(store: Store<T>, val: T, keyName?: string): QueryRoot {
     this.checkFinished();
     const doPut = (tx: IDBTransaction) => {
-      tx.objectStore(store.name).put(val);
+      const req = tx.objectStore(store.name).put(val);
+      if (keyName) {
+        req.onsuccess = () => {
+            this.keys[keyName] = req.result;
+        };
+      }
     };
     this.scheduleFinish();
     this.addWork(doPut, store.name, true);
@@ -650,13 +673,13 @@ export class QueryRoot implements PromiseLike<void> {
   /**
    * Get, modify and store an element inside a transaction.
    */
-  mutate<T>(store: Store<T>, key: any, f: (v: T) => T): QueryRoot {
+  mutate<T>(store: Store<T>, key: any, f: (v: T|undefined) => T|undefined): QueryRoot {
     this.checkFinished();
     const doPut = (tx: IDBTransaction) => {
       const reqGet = tx.objectStore(store.name).get(key);
       reqGet.onsuccess = () => {
         const r = reqGet.result;
-        let m: T;
+        let m: T|undefined;
         try {
           m = f(r);
         } catch (e) {
@@ -666,8 +689,9 @@ export class QueryRoot implements PromiseLike<void> {
           }
           throw e;
         }
-
-        tx.objectStore(store.name).put(m);
+        if (m !== undefined && m !== null) {
+          tx.objectStore(store.name).put(m);
+        }
       };
     };
     this.scheduleFinish();
