@@ -84,48 +84,64 @@ export interface ReserveRecord {
    * The reserve public key.
    */
   reserve_pub: string;
+
   /**
    * The reserve private key.
    */
   reserve_priv: string;
+
   /**
    * The exchange base URL.
    */
   exchange_base_url: string;
+
   /**
    * Time when the reserve was created.
    */
   created: number;
+
   /**
-   * Time when the reserve was last queried,
-   * or 'null' if it was never queried.
+   * Time when the reserve was depleted.
+   * Set to 0 if not depleted yet.
    */
-  last_query: number | null;
+  timestamp_depleted: number;
+
+  /**
+   * Time when the reserve was confirmed.
+   *
+   * Set to 0 if not confirmed yet.
+   */
+  timestamp_confirmed: number;
+
   /**
    * Current amount left in the reserve
    */
   current_amount: AmountJson | null;
+
   /**
    * Amount requested when the reserve was created.
    * When a reserve is re-used (rare!)  the current_amount can
    * be higher than the requested_amount
    */
   requested_amount: AmountJson;
+
   /**
    * What's the current amount that sits
    * in precoins?
    */
   precoin_amount: AmountJson;
-  /**
-   * The bank conformed that the reserve will eventually
-   * be filled with money.
-   */
-  confirmed: boolean;
+
   /**
    * We got some payback to this reserve.  We'll cease to automatically
    * withdraw money from it.
    */
   hasPayback: boolean;
+
+  /**
+   * Wire information for the bank account that
+   * transfered funds for this reserve.
+   */
+  senderWire?: object;
 }
 
 
@@ -753,6 +769,11 @@ export interface RefreshSessionRecord {
    * Is this session finished?
    */
   finished: boolean;
+
+  /**
+   * Record ID when retrieved from the DB.
+   */
+  id?: number;
 }
 
 
@@ -792,9 +813,9 @@ export enum CoinStatus {
    */
   Fresh,
   /**
-   * Currently planned to be sent to a merchant for a transaction.
+   * Currently planned to be sent to a merchant for a purchase.
    */
-  TransactionPending,
+  PurchasePending,
   /**
    * Used for a completed transaction and now dirty.
    */
@@ -811,6 +832,64 @@ export enum CoinStatus {
    * Coin fully paid back.
    */
   PaybackDone,
+  /**
+   * Coin was dirty but can't be refreshed.
+   */
+  Useless,
+}
+
+
+/**
+ * State of returning a list of coins
+ * to the customer's bank account.
+ */
+export interface CoinsReturnRecord {
+  /**
+   * Coins that we're returning.
+   */
+  coins: CoinPaySig[];
+
+  /**
+   * Responses to the deposit requests.
+   */
+  responses: any;
+
+  /**
+   * Ephemeral dummy merchant key for
+   * the coins returns operation.
+   */
+  dummyMerchantPub: string;
+
+  /**
+   * Ephemeral dummy merchant key for
+   * the coins returns operation.
+   */
+  dummyMerchantPriv: string;
+
+  /**
+   * Contract terms.
+   */
+  contractTerms: string;
+
+  /**
+   * Hash of contract terms.
+   */
+  contractTermsHash: string;
+
+  /**
+   * Wire info to send the money for the coins to.
+   */
+  wire: object;
+
+  /**
+   * Hash of the wire object.
+   */
+  wireHash: string;
+
+  /**
+   * All coins were deposited.
+   */
+  finished: boolean;
 }
 
 
@@ -903,14 +982,19 @@ export class ExchangeHandle {
 
 
 /**
- * Mapping from currency names to detailed balance
- * information for that particular currency.
+ * Mapping from currency/exchange to detailed balance
+ * information.
  */
 export interface WalletBalance {
   /**
-   * Mapping from currency name to defailed balance info.
+   * Mapping from currency name to detailed balance info.
    */
-  [currency: string]: WalletBalanceEntry;
+  byExchange: { [exchangeBaseUrl: string]: WalletBalanceEntry };
+
+  /**
+   * Mapping from currency name to detailed balance info.
+   */
+  byCurrency: { [currency: string]: WalletBalanceEntry };
 }
 
 
@@ -942,8 +1026,8 @@ export interface WalletBalanceEntry {
  */
 @Checkable.Class({validate: true})
 export class ContractTerms {
-  validate() {
-    if (this.exchanges.length === 0) {
+  static validate(x: ContractTerms) {
+    if (x.exchanges.length === 0) {
       throw Error("no exchanges in contract terms");
     }
   }
@@ -1113,6 +1197,13 @@ export class ProposalRecord {
   @Checkable.Optional(Checkable.Number)
   id?: number;
 
+  /**
+   * Timestamp (in ms) of when the record
+   * was created.
+   */
+  @Checkable.Number
+  timestamp: number;
+
   /**
    * Verify that a value matches the schema of this class and convert it into a
    * member.
@@ -1241,7 +1332,7 @@ export namespace Amounts {
       }
 
       value = value + x.value + Math.floor((fraction + x.fraction) / fractionalBase);
-      fraction = (fraction + x.fraction) % fractionalBase;
+      fraction = Math.floor((fraction + x.fraction) % fractionalBase);
       if (value > Number.MAX_SAFE_INTEGER) {
         return { amount: getMaxAmount(currency), saturated: true };
       }
@@ -1361,6 +1452,25 @@ export namespace Amounts {
       value: Number.parseInt(res[2]),
     };
   }
+
+  /**
+   * Convert the amount to a float.
+   */
+  export function toFloat(a: AmountJson): number {
+    return a.value + (a.fraction / fractionalBase);
+  }
+
+  /**
+   * Convert a float to a Taler amount.
+   * Loss of precision possible.
+   */
+  export function fromFloat(floatVal: number, currency: string) {
+    return {
+      currency,
+      fraction: Math.floor((floatVal - Math.floor(floatVal)) * fractionalBase),
+      value: Math.floor(floatVal),
+    };
+  }
 }
 
 
@@ -1384,7 +1494,10 @@ export function mkAmount(value: number, fraction: number, currency: string): Amo
 /**
  * Possible results for checkPay.
  */
-export type CheckPayResult = "paid" | "payment-possible" | "insufficient-balance";
+export interface CheckPayResult {
+  status: "paid" | "payment-possible" | "insufficient-balance";
+  coinSelection?: CoinSelectionResult;
+}
 
 /**
  * Possible results for confirmPay.
@@ -1393,18 +1506,6 @@ export type ConfirmPayResult = "paid" | "insufficient-balance";
 
 
 /**
- * Level of detail at which a history
- * entry should be shown.
- */
-export enum HistoryLevel {
-  Trace = 1,
-  Developer = 2,
-  Expert = 3,
-  User = 4,
-}
-
-
-/*
  * Activity history record.
  */
 export interface HistoryRecord {
@@ -1428,11 +1529,6 @@ export interface HistoryRecord {
    * Details used when rendering the history record.
    */
   detail: any;
-
-  /**
-   * Level of detail of the history entry.
-   */
-  level: HistoryLevel;
 }
 
 
@@ -1457,7 +1553,7 @@ export interface PayReq {
   order_id: string;
 
   /**
-   * Exchange that the coins are from.
+   * Exchange that the coins are from (base URL).
    */
   exchange: string;
 }
@@ -1484,3 +1580,205 @@ export interface QueryPaymentFound {
   contractTerms: ContractTerms;
   payReq: PayReq;
 }
+
+/**
+ * Information about all sender wire details known to the wallet,
+ * as well as exchanges that accept these wire types.
+ */
+export interface SenderWireInfos {
+  /**
+   * Mapping from exchange base url to list of accepted
+   * wire types.
+   */
+  exchangeWireTypes: { [exchangeBaseUrl: string]: string[] };
+
+  /**
+   * Sender wire types stored in the wallet.
+   */
+  senderWires: object[];
+}
+
+
+/**
+ * Request to mark a reserve as confirmed.
+ */
+@Checkable.Class()
+export class CreateReserveRequest {
+  /**
+   * The initial amount for the reserve.
+   */
+  @Checkable.Value(AmountJson)
+  amount: AmountJson;
+
+  /**
+   * Exchange URL where the bank should create the reserve.
+   */
+  @Checkable.String
+  exchange: string;
+
+  /**
+   * Wire details for the bank account that sent the funds to the exchange.
+   */
+  @Checkable.Optional(Checkable.Any)
+  senderWire?: object;
+
+  /**
+   * Verify that a value matches the schema of this class and convert it into a
+   * member.
+   */
+  static checked: (obj: any) => CreateReserveRequest;
+}
+
+
+/**
+ * Request to mark a reserve as confirmed.
+ */
+@Checkable.Class()
+export class ConfirmReserveRequest {
+  /**
+   * Public key of then reserve that should be marked
+   * as confirmed.
+   */
+  @Checkable.String
+  reservePub: string;
+
+  /**
+   * Verify that a value matches the schema of this class and convert it into a
+   * member.
+   */
+  static checked: (obj: any) => ConfirmReserveRequest;
+}
+
+
+/**
+ * Wire coins to the user's own bank account.
+ */
+@Checkable.Class()
+export class ReturnCoinsRequest {
+  /**
+   * The amount to wire.
+   */
+  @Checkable.Value(AmountJson)
+  amount: AmountJson;
+
+  /**
+   * The exchange to take the coins from.
+   */
+  @Checkable.String
+  exchange: string;
+
+  /**
+   * Wire details for the bank account of the customer that will
+   * receive the funds.
+   */
+  @Checkable.Any
+  senderWire?: object;
+
+  /**
+   * Verify that a value matches the schema of this class and convert it into a
+   * member.
+   */
+  static checked: (obj: any) => ReturnCoinsRequest;
+}
+
+
+/**
+ * Refund permission in the format that the merchant gives it to us.
+ */
+export interface RefundPermission {
+  /**
+   * Amount to be refunded.
+   */
+  refund_amount: AmountJson;
+
+  /**
+   * Fee for the refund.
+   */
+  refund_fee: AmountJson;
+
+  /**
+   * Contract terms hash to identify the contract that this
+   * refund is for.
+   */
+  h_contract_terms: string;
+
+  /**
+   * Public key of the coin being refunded.
+   */
+  coin_pub: string;
+
+  /**
+   * Refund transaction ID between merchant and exchange.
+   */
+  rtransaction_id: number;
+
+  /**
+   * Public key of the merchant.
+   */
+  merchant_pub: string;
+
+  /**
+   * Signature made by the merchant over the refund permission.
+   */
+  merchant_sig: string;
+}
+
+
+/**
+ * Record that stores status information about one purchase, starting from when
+ * the customer accepts a proposal.  Includes refund status if applicable.
+ */
+export interface PurchaseRecord {
+  contractTermsHash: string;
+  contractTerms: ContractTerms;
+  payReq: PayReq;
+  merchantSig: string;
+
+  /**
+   * The purchase isn't active anymore, it's either successfully paid or
+   * refunded/aborted.
+   */
+  finished: boolean;
+
+  refundsPending: { [refundSig: string]: RefundPermission };
+  refundsDone: { [refundSig: string]: RefundPermission };
+
+  /**
+   * When was the purchase made?
+   * Refers to the time that the user accepted.
+   */
+  timestamp: number;
+
+  /**
+   * When was the last refund made?
+   * Set to 0 if no refund was made on the purchase.
+   */
+  timestamp_refund: number;
+}
+
+
+/**
+ * Result of selecting coins, contains the exchange, and selected
+ * coins with their denomination.
+ */
+export interface CoinSelectionResult {
+  exchangeUrl: string;
+  cds: CoinWithDenom[];
+  totalFees: AmountJson;
+}
+
+
+/**
+ * Named tuple of coin and denomination.
+ */
+export interface CoinWithDenom {
+  /**
+   * A coin.  Must have the same denomination public key as the associated
+   * denomination.
+   */
+  coin: CoinRecord;
+  /**
+   * An associated denomination.
+   */
+  denom: DenominationRecord;
+}
