@@ -35,17 +35,18 @@ import {
 
 import {ImplicitStateComponent, StateHolder} from "../components";
 import {
+  createReserve,
   getCurrency,
   getExchangeInfo,
   getReserveCreationInfo,
 } from "../wxApi";
 
-import {renderAmount} from "../renderHtml";
+import {Collapsible, renderAmount} from "../renderHtml";
 
+import * as moment from "moment";
 import * as React from "react";
 import * as ReactDOM from "react-dom";
 import URI = require("urijs");
-import * as moment from "moment";
 
 
 function delay<T>(delayMs: number, value: T): Promise<T> {
@@ -79,41 +80,6 @@ class EventTrigger {
 }
 
 
-interface CollapsibleState {
-  collapsed: boolean;
-}
-
-interface CollapsibleProps {
-  initiallyCollapsed: boolean;
-  title: string;
-}
-
-class Collapsible extends React.Component<CollapsibleProps, CollapsibleState> {
-  constructor(props: CollapsibleProps) {
-    super(props);
-    this.state = { collapsed: props.initiallyCollapsed };
-  }
-  render() {
-    const doOpen = (e: any) => {
-      this.setState({collapsed: false});
-      e.preventDefault();
-    };
-    const doClose = (e: any) => {
-      this.setState({collapsed: true});
-      e.preventDefault();
-    };
-    if (this.state.collapsed) {
-      return <h2><a className="opener opener-collapsed" href="#" onClick={doOpen}>{this.props.title}</a></h2>;
-    }
-    return (
-      <div>
-        <h2><a className="opener opener-open" href="#" onClick={doClose}>{this.props.title}</a></h2>
-        {this.props.children}
-      </div>
-    );
-  }
-}
-
 function renderAuditorDetails(rci: ReserveCreationInfo|null) {
   console.log("rci", rci);
   if (!rci) {
@@ -242,6 +208,7 @@ interface ExchangeSelectionProps {
   callback_url: string;
   wt_types: string[];
   currencyRecord: CurrencyRecord|null;
+  sender_wire: object | undefined;
 }
 
 interface ManualSelectionProps {
@@ -403,7 +370,7 @@ class ExchangeSelection extends ImplicitStateComponent<ExchangeSelectionProps> {
     if (this.statusString()) {
       return (
         <p>
-          <strong style={{color: "red"}}>{i18n.str`A problem occured, see below. ${this.statusString()}`}</strong>
+          <strong style={{color: "red"}}>{this.statusString()}</strong>
         </p>
       );
     }
@@ -425,7 +392,7 @@ class ExchangeSelection extends ImplicitStateComponent<ExchangeSelectionProps> {
     if (rci.versionMatch.compatible) {
       return null;
     }
-    if (rci.versionMatch.currentCmp == -1) {
+    if (rci.versionMatch.currentCmp === -1) {
       return (
         <p className="errorbox">
           Your wallet might be outdated.  The exchange has a higher, incompatible
@@ -433,7 +400,7 @@ class ExchangeSelection extends ImplicitStateComponent<ExchangeSelectionProps> {
         </p>
       );
     }
-    if (rci.versionMatch.currentCmp == 1) {
+    if (rci.versionMatch.currentCmp === 1) {
       return (
         <p className="errorbox">
           The chosen exchange might be outdated.  The exchange has a lower, incompatible
@@ -529,7 +496,8 @@ class ExchangeSelection extends ImplicitStateComponent<ExchangeSelectionProps> {
     this.confirmReserveImpl(this.reserveCreationInfo()!,
                             this.url()!,
                             this.props.amount,
-                            this.props.callback_url);
+                            this.props.callback_url,
+                            this.props.sender_wire);
   }
 
   /**
@@ -546,62 +514,61 @@ class ExchangeSelection extends ImplicitStateComponent<ExchangeSelectionProps> {
       console.dir(r);
     } catch (e) {
       console.log("get exchange info rejected", e);
-      if (e.hasOwnProperty("httpStatus")) {
-        this.statusString(`Error: request failed with status ${e.httpStatus}`);
-      } else if (e.hasOwnProperty("errorResponse")) {
-        const resp = e.errorResponse;
-        this.statusString(`Error: ${resp.error} (${resp.hint})`);
-      }
+      this.statusString(`Error: ${e.message}`);
+      // Re-try every 5 seconds as long as there is a problem
+      setTimeout(() => this.statusString() ? this.forceReserveUpdate() : undefined, 5000);
     }
   }
 
-  confirmReserveImpl(rci: ReserveCreationInfo,
-                     exchange: string,
-                     amount: AmountJson,
-                     callback_url: string) {
-    const d = {exchange: canonicalizeBaseUrl(exchange), amount};
-    const cb = (rawResp: any) => {
-      if (!rawResp) {
-        throw Error("empty response");
+  async confirmReserveImpl(rci: ReserveCreationInfo,
+                           exchange: string,
+                           amount: AmountJson,
+                           callback_url: string,
+                           sender_wire: object | undefined) {
+    const rawResp = await createReserve({
+      amount,
+      exchange: canonicalizeBaseUrl(exchange),
+      senderWire: sender_wire,
+    });
+    if (!rawResp) {
+      throw Error("empty response");
+    }
+    // FIXME: filter out types that bank/exchange don't have in common
+    const wireDetails = rci.wireInfo;
+    const filteredWireDetails: any = {};
+    for (const wireType in wireDetails) {
+      if (this.props.wt_types.findIndex((x) => x.toLowerCase() === wireType.toLowerCase()) < 0) {
+        continue;
       }
-      // FIXME: filter out types that bank/exchange don't have in common
-      const wireDetails = rci.wireInfo;
-      const filteredWireDetails: any = {};
-      for (const wireType in wireDetails) {
-        if (this.props.wt_types.findIndex((x) => x.toLowerCase() === wireType.toLowerCase()) < 0) {
-          continue;
-        }
-        const obj = Object.assign({}, wireDetails[wireType]);
-        // The bank doesn't need to know about fees
-        delete obj.fees;
-        // Consequently the bank can't verify signatures anyway, so
-        // we delete this extra data, to make the request URL shorter.
-        delete obj.salt;
-        delete obj.sig;
-        filteredWireDetails[wireType] = obj;
+      const obj = Object.assign({}, wireDetails[wireType]);
+      // The bank doesn't need to know about fees
+      delete obj.fees;
+      // Consequently the bank can't verify signatures anyway, so
+      // we delete this extra data, to make the request URL shorter.
+      delete obj.salt;
+      delete obj.sig;
+      filteredWireDetails[wireType] = obj;
+    }
+    if (!rawResp.error) {
+      const resp = CreateReserveResponse.checked(rawResp);
+      const q: {[name: string]: string|number} = {
+        amount_currency: amount.currency,
+        amount_fraction: amount.fraction,
+        amount_value: amount.value,
+        exchange: resp.exchange,
+        reserve_pub: resp.reservePub,
+        wire_details: JSON.stringify(filteredWireDetails),
+      };
+      const url = new URI(callback_url).addQuery(q);
+      if (!url.is("absolute")) {
+        throw Error("callback url is not absolute");
       }
-      if (!rawResp.error) {
-        const resp = CreateReserveResponse.checked(rawResp);
-        const q: {[name: string]: string|number} = {
-          amount_currency: amount.currency,
-          amount_fraction: amount.fraction,
-          amount_value: amount.value,
-          exchange: resp.exchange,
-          reserve_pub: resp.reservePub,
-          wire_details: JSON.stringify(filteredWireDetails),
-        };
-        const url = new URI(callback_url).addQuery(q);
-        if (!url.is("absolute")) {
-          throw Error("callback url is not absolute");
-        }
-        console.log("going to", url.href());
-        document.location.href = url.href();
-      } else {
-        this.statusString(
-          i18n.str`Oops, something went wrong. The wallet responded with error status (${rawResp.error}).`);
-      }
-    };
-    chrome.runtime.sendMessage({type: "create-reserve", detail: d}, cb);
+      console.log("going to", url.href());
+      document.location.href = url.href();
+    } else {
+      this.statusString(
+        i18n.str`Oops, something went wrong. The wallet responded with error status (${rawResp.error}).`);
+    }
   }
 
   renderStatus(): any {
@@ -632,6 +599,11 @@ async function main() {
       throw Error(i18n.str`Can't parse wire_types: ${e.message}`);
     }
 
+    let sender_wire;
+    if (query.sender_wire) {
+      sender_wire = JSON.parse(query.sender_wire);
+    }
+
     const suggestedExchangeUrl = query.suggested_exchange_url;
     const currencyRecord = await getCurrency(amount.currency);
 
@@ -639,6 +611,7 @@ async function main() {
       amount,
       callback_url,
       currencyRecord,
+      sender_wire,
       suggestedExchangeUrl,
       wt_types,
     };
