@@ -32,12 +32,13 @@ import {
 } from "../query";
 import {
   AmountJson,
-  Notifier,
-  ProposalRecord,
-} from "../types";
-import {
   ConfirmReserveRequest,
   CreateReserveRequest,
+  Notifier,
+  ProposalRecord,
+  ReturnCoinsRequest,
+} from "../types";
+import {
   Stores,
   WALLET_DB_VERSION,
   Wallet,
@@ -114,6 +115,7 @@ function handleMessage(sender: MessageSender,
       const d = {
         amount: detail.amount,
         exchange: detail.exchange,
+        senderWire: detail.senderWire,
       };
       const req = CreateReserveRequest.checked(d);
       return needsWallet().createReserve(req);
@@ -175,19 +177,9 @@ function handleMessage(sender: MessageSender,
         return hash;
       });
     }
-    case "put-history-entry": {
-      if (!detail.historyEntry) {
-        return Promise.resolve({ error: "historyEntry missing" });
-      }
-      return needsWallet().putHistory(detail.historyEntry);
-    }
     case "save-proposal": {
       console.log("handling save-proposal", detail);
-      const checkedRecord = ProposalRecord.checked({
-        contractTerms: detail.data,
-        contractTermsHash: detail.hash,
-        merchantSig: detail.sig,
-      });
+      const checkedRecord = ProposalRecord.checked(detail.proposal);
       return needsWallet().saveProposal(checkedRecord);
     }
     case "reserve-creation-info": {
@@ -238,7 +230,7 @@ function handleMessage(sender: MessageSender,
       if (typeof detail.exchangeBaseUrl !== "string") {
         return Promise.reject(Error("exchangBaseUrl missing"));
       }
-      return needsWallet().getPreCoins(detail.exchangeBaseUrl); 
+      return needsWallet().getPreCoins(detail.exchangeBaseUrl);
     }
     case "get-denoms": {
       if (typeof detail.exchangeBaseUrl !== "string") {
@@ -277,18 +269,53 @@ function handleMessage(sender: MessageSender,
       }
       return needsWallet().paymentSucceeded(contractTermsHash, merchantSig);
     }
+    case "get-sender-wire-infos": {
+      return needsWallet().getSenderWireInfos();
+    }
+    case "return-coins": {
+      const d = {
+        amount: detail.amount,
+        exchange: detail.exchange,
+        senderWire: detail.senderWire,
+      };
+      const req = ReturnCoinsRequest.checked(d);
+      return needsWallet().returnCoins(req);
+    }
     case "check-upgrade": {
       let dbResetRequired = false;
       if (!currentWallet) {
         dbResetRequired = true;
       }
       const resp: wxApi.UpgradeResponse = {
-        dbResetRequired,
         currentDbVersion: WALLET_DB_VERSION.toString(),
+        dbResetRequired,
         oldDbVersion: (oldDbVersion || "unknown").toString(),
-      }
+      };
       return resp;
     }
+    case "log-and-display-error":
+      logging.storeReport(detail).then((reportUid) => {
+        const url = chrome.extension.getURL(`/src/webex/pages/error.html?reportUid=${reportUid}`);
+        if (detail.sameTab && sender && sender.tab && sender.tab.id) {
+          chrome.tabs.update(detail.tabId, { url });
+        } else {
+          chrome.tabs.create({ url });
+        }
+      });
+      return;
+    case "get-report":
+      return logging.getReport(detail.reportUid);
+    case "accept-refund":
+      return needsWallet().acceptRefund(detail.refund_permissions);
+    case "get-purchase": {
+      const contractTermsHash = detail.contractTermsHash;
+      if (!contractTermsHash) {
+        throw Error("contractTermsHash missing");
+      }
+      return needsWallet().getPurchase(contractTermsHash);
+    }
+    case "get-full-refund-fees":
+      return needsWallet().getFullRefundFees(detail.refundPermissions);
     default:
       // Exhaustiveness check.
       // See https://www.typescriptlang.org/docs/handbook/advanced-types.html
@@ -318,7 +345,7 @@ async function dispatch(req: any, sender: any, sendResponse: any): Promise<void>
     try {
       sendResponse({
         error: "exception",
-        hint: e.message,
+        message: e.message,
         stack,
       });
     } catch (e) {
@@ -357,6 +384,9 @@ class ChromeNotifier implements Notifier {
 
 /**
  * Mapping from tab ID to payment information (if any).
+ *
+ * Used to pass information from an intercepted HTTP header to the content
+ * script on the page.
  */
 const paymentRequestCookies: { [n: number]: any } = {};
 
@@ -378,6 +408,7 @@ function handleHttpPayment(headerList: chrome.webRequest.HttpHeader[], url: stri
   const fields = {
     contract_url: headers["x-taler-contract-url"],
     offer_url: headers["x-taler-offer-url"],
+    refund_url: headers["x-taler-refund-url"],
   };
 
   const talerHeaderFound = Object.keys(fields).filter((x: any) => (fields as any)[x]).length !== 0;
@@ -392,6 +423,7 @@ function handleHttpPayment(headerList: chrome.webRequest.HttpHeader[], url: stri
   const payDetail = {
     contract_url: fields.contract_url,
     offer_url: fields.offer_url,
+    refund_url: fields.refund_url,
   };
 
   console.log("got pay detail", payDetail);
@@ -414,15 +446,29 @@ function handleBankRequest(wallet: Wallet, headerList: chrome.webRequest.HttpHea
     }
   }
 
-  const reservePub = headers["x-taler-reserve-pub"];
-  if (reservePub !== undefined) {
-    console.log(`confirming reserve ${reservePub} via 201`);
-    wallet.confirmReserve({reservePub});
+  const operation = headers["x-taler-operation"];
+
+  if (!operation) {
+    // Not a taler related request.
     return;
   }
 
-  const amount = headers["x-taler-amount"];
-  if (amount) {
+  if (operation === "confirm-reserve") {
+    const reservePub = headers["x-taler-reserve-pub"];
+    if (reservePub !== undefined) {
+      console.log(`confirming reserve ${reservePub} via 201`);
+      wallet.confirmReserve({reservePub});
+    }
+    console.warn("got 'X-Taler-Operation: confirm-reserve' without 'X-Taler-Reserve-Pub'");
+    return;
+  }
+
+  if (operation === "create-reserve") {
+    const amount = headers["x-taler-amount"];
+    if (!amount) {
+      console.log("202 not understood (X-Taler-Amount missing)");
+      return;
+    }
     const callbackUrl = headers["x-taler-callback-url"];
     if (!callbackUrl) {
       console.log("202 not understood (X-Taler-Callback-Url missing)");
@@ -432,13 +478,13 @@ function handleBankRequest(wallet: Wallet, headerList: chrome.webRequest.HttpHea
     try {
       amountParsed = JSON.parse(amount);
     } catch (e) {
-      const uri = new URI(chrome.extension.getURL("/src/webex/pages/error.html"));
+      const errUri = new URI(chrome.extension.getURL("/src/webex/pages/error.html"));
       const p = {
         message: `Can't parse amount ("${amount}"): ${e.message}`,
       };
-      const redirectUrl = uri.query(p).href();
+      const errRedirectUrl = errUri.query(p).href();
       // FIXME: use direct redirect when https://bugzilla.mozilla.org/show_bug.cgi?id=707624 is fixed
-      chrome.tabs.update(tabId, {url: redirectUrl});
+      chrome.tabs.update(tabId, {url: errRedirectUrl});
       return;
     }
     const wtTypes = headers["x-taler-wt-types"];
@@ -450,6 +496,7 @@ function handleBankRequest(wallet: Wallet, headerList: chrome.webRequest.HttpHea
       amount,
       bank_url: url,
       callback_url: new URI(callbackUrl) .absoluteTo(url),
+      sender_wire: headers["x-taler-sender-wire"],
       suggested_exchange_url: headers["x-taler-suggested-exchange"],
       wt_types: wtTypes,
     };
@@ -460,7 +507,8 @@ function handleBankRequest(wallet: Wallet, headerList: chrome.webRequest.HttpHea
     chrome.tabs.update(tabId, {url: redirectUrl});
     return;
   }
-  // no known headers found, not a taler request ...
+
+  console.log("Ignoring unknown X-Taler-Operation:", operation);
 }
 
 
@@ -508,6 +556,24 @@ async function reinitWallet() {
 }
 
 
+/**
+ * Inject a script into a tab.  Gracefully logs errors
+ * and works around a bug where the tab's URL does not match the internal URL,
+ * making the injection fail in a confusing way.
+ */
+function injectScript(tabId: number, details: chrome.tabs.InjectDetails, actualUrl: string): void {
+  chrome.tabs.executeScript(tabId, details,  () => {
+    // Required to squelch chrome's "unchecked lastError" warning.
+    // Sometimes chrome reports the URL of a tab as http/https but
+    // injection fails.  This can happen when a page is unloaded or
+    // shows a "no internet" page etc.
+    if (chrome.runtime.lastError) {
+      console.warn("injection failed on page", actualUrl, chrome.runtime.lastError.message);
+    }
+  });
+}
+
+
 /**
  * Main function to run for the WebExtension backend.
  *
@@ -519,28 +585,30 @@ export async function wxMain() {
   chrome.runtime.onUpdateAvailable.addListener((details) => {
     console.log("update available:", details);
     chrome.runtime.reload();
-  })
+  });
 
   window.onerror = (m, source, lineno, colno, error) => {
     logging.record("error", m + error, undefined, source || "(unknown)", lineno || 0, colno || 0);
   };
 
   chrome.tabs.query({}, (tabs) => {
+    console.log("got tabs", tabs);
     for (const tab of tabs) {
       if (!tab.url || !tab.id) {
-        return;
+        continue;
       }
       const uri = new URI(tab.url);
-      if (uri.protocol() === "http" || uri.protocol() === "https") {
-        console.log("injecting into existing tab", tab.id);
-        chrome.tabs.executeScript(tab.id, { file: "/dist/contentScript-bundle.js" });
-        const code = `
-          if (("taler" in window) || document.documentElement.getAttribute("data-taler-nojs")) {
-            document.dispatchEvent(new Event("taler-probe-result"));
-          }
-        `;
-        chrome.tabs.executeScript(tab.id, { code, runAt: "document_idle" });
+      if (uri.protocol() !== "http" && uri.protocol() !== "https") {
+        continue;
       }
+      console.log("injecting into existing tab", tab.id, "with url", uri.href(), "protocol", uri.protocol());
+      injectScript(tab.id, { file: "/dist/contentScript-bundle.js", runAt: "document_start" }, uri.href());
+      const code = `
+        if (("taler" in window) || document.documentElement.getAttribute("data-taler-nojs")) {
+          document.dispatchEvent(new Event("taler-probe-result"));
+        }
+      `;
+      injectScript(tab.id, { code, runAt: "document_start" }, uri.href());
     }
   });
 
@@ -581,7 +649,7 @@ export async function wxMain() {
             document.dispatchEvent(new Event("taler-probe-result"));
           }
         `;
-        chrome.tabs.executeScript(tab.id!, { code, runAt: "document_start" });
+        injectScript(tab.id!, { code, runAt: "document_start" }, uri.href());
       });
     };
 
